@@ -1,6 +1,6 @@
 # pylint:disable=abstract-method
 
-from typing import Set
+from typing import Set, List
 import logging
 
 import pyvex
@@ -488,18 +488,36 @@ class StackPointerTracker(Analysis, ForwardAnalysis):
                         pass
 
             # stack pointer adjustment
-            if self.project.arch.sp_offset in self.reg_offsets \
-                    and vex_block.jumpkind == 'Ijk_Call' \
-                    and self.project.arch.call_pushes_ret:
-                try:
-                    v = state.get(self.project.arch.sp_offset)
-                    if v is BOTTOM:
-                        incremented = BOTTOM
-                    else:
-                        incremented = v + Constant(self.project.arch.bytes)
-                    state.put(self.project.arch.sp_offset, incremented)
-                except CouldNotResolveException:
-                    pass
+            if self.project.arch.sp_offset in self.reg_offsets and vex_block.jumpkind == 'Ijk_Call':
+                if self.project.arch.call_pushes_ret:
+                    # pop the return address on the stack
+                    try:
+                        v = state.get(self.project.arch.sp_offset)
+                        if v is BOTTOM:
+                            incremented = BOTTOM
+                        else:
+                            incremented = v + Constant(self.project.arch.bytes)
+                        state.put(self.project.arch.sp_offset, incremented)
+                    except CouldNotResolveException:
+                        pass
+                # who are we calling?
+                callees = self._find_callees(node)
+                if callees:
+                    callee_cleanups = [callee for callee in callees if callee.calling_convention is not None and
+                                       callee.calling_convention.args is not None and
+                                       callee.calling_convention.CALLEE_CLEANUP]
+                    if callee_cleanups:
+                        # found callee clean-up cases...
+                        try:
+                            v = state.get(self.project.arch.sp_offset)
+                            if v is BOTTOM:
+                                incremented = BOTTOM
+                            else:
+                                num_args = len(callee_cleanups[0].calling_convention.args)
+                                incremented = v + Constant(self.project.arch.bytes * num_args)
+                            state.put(self.project.arch.sp_offset, incremented)
+                        except CouldNotResolveException:
+                            pass
 
         if curr_stmt_start_addr is not None:
             self._set_post_state(curr_stmt_start_addr, state.freeze())
@@ -526,5 +544,13 @@ class StackPointerTracker(Analysis, ForwardAnalysis):
             merged_state = merged_state.merge(other)
         return merged_state, merged_state == states[0]
 
+    def _find_callees(self, node) -> List[Function]:
+        callees: List[Function] = [ ]
+        for _, dst, data in self._func.transition_graph.out_edges(node, data=True):
+            if data.get('type') == 'call':
+                if isinstance(dst, Function):
+                    callees.append(dst)
+        return callees
+
 
 AnalysesHub.register_default('StackPointerTracker', StackPointerTracker)
