@@ -12,9 +12,12 @@ def obj_bit_size(o):
 
 class SimMemoryObject:
     """
-    A MemoryObjectRef instance is a reference to a byte or several bytes in
-    a specific object in memory. It should be used only by the bottom layer of memory.
+    A SimMemoryObject is a reference to a byte or several bytes in a specific object in memory. It should be used only
+    by the bottom layer of memory.
     """
+
+    __slots__ = ('is_bytes', '_byte_width', 'base', 'object', 'length', 'endness', )
+
     def __init__(self, obj, base, endness, length=None, byte_width=8):
         if type(obj) is bytes:
             assert byte_width == 8
@@ -22,7 +25,7 @@ class SimMemoryObject:
         elif not isinstance(obj, claripy.ast.Base):
             raise SimMemoryError('memory can only store claripy Expression')
 
-        self.is_bytes = type(obj) == bytes
+        self.is_bytes = type(obj) is bytes
         if self.is_bytes and endness != 'Iend_BE':
             raise SimMemoryError('bytes can only be stored big-endian')
         self._byte_width = byte_width
@@ -110,12 +113,15 @@ class SimMemoryObject:
             return self.length.cache_key == other.length.cache_key
 
     def __eq__(self, other):
+        if self is other:
+            return True
+
         if type(other) is not SimMemoryObject:
             return NotImplemented
 
-        return  self.base == other.base and \
-                self._object_equals(other) and \
-                self._length_equals(other)
+        return self.base == other.base and \
+               self._object_equals(other) and \
+               self._length_equals(other)
 
     def __hash__(self):
         obj_hash = hash(self.object) if self.is_bytes else self.object.cache_key
@@ -127,6 +133,16 @@ class SimMemoryObject:
     def __repr__(self):
         return "MO(%s)" % self.object
 
+
+class SimLabeledMemoryObject(SimMemoryObject):
+
+    __slots__ = ('label', )
+
+    def __init__(self, obj, base, endness, length=None, byte_width=8, label=None):
+        super().__init__(obj, base, endness, length=length, byte_width=byte_width)
+        self.label = label
+
+
 def bv_slice(value, offset, size, rev, bw):
     """
     Extremely cute utility to pretend you've serialized a value to stored bytes, sliced it a la python slicing, and then
